@@ -1,32 +1,32 @@
-from dataclasses import dataclass, field
-from token import OP
-from typing import List, Optional, Dict, TypeVar, Any
-import uuid
-import time
+import copy
+import hashlib
 import json
 import os
-import hashlib
-import copy
+import time
+import uuid
+from typing import Any, Dict, List, Optional, TypeVar
 
 import requests
+import shortuuid
+from PIL import Image
 from sqlalchemy import asc
 
-from threadmem.db.models import RoleMessageRecord, RoleThreadRecord
 from threadmem.db.conn import WithDB
+from threadmem.db.models import RoleMessageRecord, RoleThreadRecord
+
+from .env import HUB_API_KEY_ENV
+from .img import convert_images, image_to_b64
 from .server.models import (
+    V1Role,
     V1RoleMessage,
     V1RoleThread,
-    V1UpdateRoleThread,
     V1RoleThreads,
-    V1Role,
+    V1UpdateRoleThread,
 )
-from .env import HUB_API_KEY_ENV
-
 
 R = TypeVar("R", bound="RoleThread")
 
 
-@dataclass
 class RoleMessage(WithDB):
     """
     A role-based chat message that supports text, optional images, and metadata.
@@ -39,21 +39,27 @@ class RoleMessage(WithDB):
         thread_id (str): The ID of the thread this message belongs to.
         images (List[str]): A list of image URLs associated with the message. Defaults to an empty list.
         private (Optional[bool]): A flag indicating if the message is private. Defaults to False.
-        created (float): The timestamp of when the message was created. Defaults to the current time.
-        id (str): A unique identifier for the message. Defaults to a UUID4 string.
         metadata (Optional[dict]): Additional metadata associated with the message. Defaults to None.
     """
 
-    role: str
-    text: str
-    thread_id: Optional[str] = None
-    images: List[str] = field(default_factory=list)
-    private: Optional[bool] = False
-    created: float = field(default_factory=time.time)
-    id: str = field(default_factory=lambda: str(uuid.uuid4()))
-    metadata: Optional[dict] = None
+    def __init__(
+        self,
+        role: str,
+        text: str,
+        thread_id: Optional[str] = None,
+        images: List[str] = [],
+        private: Optional[bool] = False,
+        metadata: Optional[dict] = None,
+    ) -> None:
+        self.role = role
+        self.text = text
+        self.thread_id = thread_id
+        self.images = convert_images(images) if images else []  # type: ignore
+        self.private = private
+        self.created = time.time()
+        self.id = shortuuid.uuid()
+        self.metadata = metadata
 
-    def __post_init__(self) -> None:
         self.save()
 
     @classmethod
@@ -80,20 +86,39 @@ class RoleMessage(WithDB):
         """Converts a RoleMessage to the format expected by OpenAI."""
         content = []
 
-        # Add text content if it exists
+        # Split text by <image> tags and process each part
         if self.text:
-            content.append({"type": "text", "text": self.text})
+            parts = self.text.split("<image>")
+            image_index = 0
 
-        # Add images if they exist
-        for image_url in self.images:
-            content.append(
-                {
-                    "type": "image_url",
-                    "image_url": {
-                        "url": image_url,
-                    },
-                }
-            )
+            for i, part in enumerate(parts):
+                # Add text content if it exists (strip to remove any whitespace)
+                if part.strip():
+                    content.append({"type": "text", "text": part.strip()})
+
+                # Add the next image after each text part (except for the last split)
+                if i < len(parts) - 1 and image_index < len(self.images):
+                    content.append(
+                        {
+                            "type": "image_url",
+                            "image_url": {
+                                "url": self.images[image_index],
+                            },
+                        }
+                    )
+                    image_index += 1
+
+            # Add any remaining images
+            while image_index < len(self.images):
+                content.append(
+                    {
+                        "type": "image_url",
+                        "image_url": {
+                            "url": self.images[image_index],
+                        },
+                    }
+                )
+                image_index += 1
 
         # Assemble the final JSON structure
         return {"role": self.role, "content": content}
@@ -106,7 +131,8 @@ class RoleMessage(WithDB):
             RoleMessageRecord: An instance of RoleMessageRecord with fields populated from the RoleMessage instance.
         """
         metadata = json.dumps(self.metadata) if self.metadata else None
-        images = json.dumps(self.images) if self.images else None
+        images = json.dumps(self.images)
+
         return RoleMessageRecord(
             id=self.id,
             text=self.text,
@@ -305,7 +331,7 @@ class RoleThread(WithDB):
                     if c["type"] == "text":
                         text = c["text"]
                     elif c["type"] == "image_url":
-                        images.append(c["url"])
+                        images.append(c["image_url"]["url"])
             else:
                 text = content
 
@@ -459,7 +485,7 @@ class RoleThread(WithDB):
         self,
         role: str,
         msg: str,
-        images: List[str] = [],
+        images: List[str | Image.Image] = [],
         private: bool = False,
         metadata: Optional[dict] = None,
     ) -> None:
@@ -473,13 +499,27 @@ class RoleThread(WithDB):
         Args:
             role (str): The role associated with the message.
             msg (str): The message text.
-            images (List[str], optional): A list of image URLs. Defaults to an empty list.
+            images (List[str | Image.Image], optional): A list of images; accepts URLs, PIL images, local filepaths, or b64 encoded images.
+                Defaults to an empty list.
             private (bool, optional): Whether the message is private. Defaults to False.
             metadata (Optional[dict], optional): Additional metadata for the message. Defaults to None.
 
         Raises:
             Exception: If an error occurs while posting the message to a remote server.
         """
+        new_imgs: List[str] = []
+        for img in images:
+            if isinstance(img, Image.Image):
+                new_imgs.append(image_to_b64(img))
+            elif isinstance(img, str):
+                if img.startswith("data:") or img.startswith("http"):
+                    new_imgs.append(img)
+                else:
+                    loaded_img = Image.open(img)
+                    new_imgs.append(image_to_b64(loaded_img))
+            else:
+                raise ValueError("unnknown image type")
+
         if self._remote:
             # print("\nposting msg to remote task", self._id)
             try:
@@ -503,7 +543,7 @@ class RoleThread(WithDB):
                     role,
                     msg,
                     thread_id=self._id,
-                    images=images,
+                    images=new_imgs,
                     private=private,
                     metadata=metadata,
                 )
@@ -900,7 +940,6 @@ class RoleThread(WithDB):
         if self._remote:
             # print("\nrefreshing remote thread", self._id)
             try:
-
                 remote_thread = self._remote_request(
                     self._remote,
                     "GET",
